@@ -18,8 +18,9 @@
 #ifndef METATILE_H
 #define METATILE_H
 
-#include "config.h"
 #include <stdlib.h>
+
+#include "config.h"
 #include "render_config.h"
 
 #ifdef __cplusplus
@@ -37,14 +38,13 @@ struct entry {
 
 struct meta_layout {
 	char magic[4];
-	int count; // METATILE ^ 2
-	int x, y, z; // lowest x,y of this metatile, plus z
+	int count;	      // METATILE ^ 2
+	int x, y, z;	      // lowest x,y of this metatile, plus z
 	struct entry index[]; // count entries
 	// Followed by the tile data
 	// The index offsets are measured from the start of the file
 };
 
-
 #ifdef __cplusplus
 }
 
@@ -56,17 +56,16 @@ public:
 	void set(int x, int y, const std::string &data);
 	const std::string get(int x, int y);
 	int xyz_to_meta_offset(int x, int y, int z);
-	void save(struct storage_backend * store);
-	void expire_tiles(int sock, char * host, char * uri);
+	void save(struct storage_backend *store);
+	void expire_tiles(int sock, const char *host, const char *uri);
+
 private:
 	int x_, y_, z_;
 	std::string xmlconfig_;
 	std::string options_;
 	std::string tile[METATILE][METATILE];
 	static const int header_size = sizeof(struct meta_layout) + (sizeof(struct entry) * (METATILE * METATILE));
-
 };
 
 #endif
 #endif
-
