@@ -18,7 +18,10 @@
 #ifndef MODTILE_H
 #define MODTILE_H
 
+#include "protocol.h"
 #include "store.h"
+#include <apr_tables.h>
+#include <netinet/in.h>
 
 /*Size of the delaypool hashtable*/
 #define DELAY_HASHTABLE_SIZE 100057
@@ -43,7 +46,6 @@
 #define VERYOLD_RENDER 5
 #define VERYOLD 6
 
-
 /* Number of microseconds to camp out on the mutex */
 #define CAMPOUT 10
 /* Maximum number of times we camp out before giving up */
@@ -90,63 +92,66 @@ typedef struct stats_data {
 } stats_data;
 
 typedef struct {
-	const char * store;
-	char xmlname[XMLCONFIG_MAX];
-	char baseuri[PATH_MAX];
-	char fileExtension[PATH_MAX];
-	char mimeType[PATH_MAX];
-	const char * description;
-	const char * attribution;
-	const char * cors;
 	char **hostnames;
-	int noHostnames;
-	int minzoom;
-	int maxzoom;
+	const char *attribution;
+	const char *baseuri;
+	const char *cors;
+	const char *description;
+	const char *fileExtension;
+	const char *mimeType;
+	const char *store;
+	const char *xmlname;
 	int aspect_x;
 	int aspect_y;
 	int enableOptions;
+	int maxzoom;
+	int minzoom;
+	int noHostnames;
 } tile_config_rec;
 
 typedef struct {
 	apr_array_header_t *configs;
+	apr_time_t very_old_threshold;
+	const char *cache_extended_hostname;
+	const char *renderd_socket_name;
+	const char *tile_dir;
+	double cache_duration_last_modified_factor;
+	int cache_duration_dirty;
+	int cache_duration_low_zoom;
+	int cache_duration_max;
+	int cache_duration_medium_zoom;
+	int cache_duration_minimum;
+	int cache_extended_duration;
+	int cache_level_low_zoom;
+	int cache_level_medium_zoom;
+	int delaypool_render_size;
+	int delaypool_tile_size;
+	int enable_bulk_mode;
+	int enable_dirty_url;
+	int enable_global_stats;
+	int enable_status_url;
+	int enable_tile_throttling;
+	int enable_tile_throttling_xforward;
+	int max_load_missing;
+	int max_load_old;
+	int mincachetime[MAX_ZOOM_SERVER + 1];
+	int renderd_socket_port;
 	int request_timeout;
 	int request_timeout_priority;
-	int max_load_old;
-	int max_load_missing;
-	apr_time_t veryold_threshold;
-	int cache_duration_dirty;
-	int cache_duration_max;
-	int cache_duration_minimum;
-	int cache_duration_low_zoom;
-	int cache_level_low_zoom;
-	int cache_duration_medium_zoom;
-	int cache_level_medium_zoom;
-	double cache_duration_last_modified_factor;
-	char renderd_socket_name[PATH_MAX];
-	int renderd_socket_port;
-	char tile_dir[PATH_MAX];
-	char cache_extended_hostname[PATH_MAX];
-	int  cache_extended_duration;
-	int mincachetime[MAX_ZOOM_SERVER + 1];
-	int enableGlobalStats;
-	int enableTileThrottling;
-	int enableTileThrottlingXForward;
-	int delaypoolTileSize;
-	long delaypoolTileRate;
-	int delaypoolRenderSize;
-	long delaypoolRenderRate;
-	int bulkMode;
-	int enableStatusUrl;
-	int enableDirtyUrl;
+	long delaypool_render_rate;
+	long delaypool_tile_rate;
 } tile_server_conf;
 
 typedef struct tile_request_data {
-	struct protocol * cmd;
-	struct storage_backend * store;
+	struct protocol *cmd;
+	struct storage_backend *store;
 	int layerNumber;
 } tile_request_data;
 
-enum tileState { tileMissing, tileOld, tileVeryOld, tileCurrent };
-
+enum tileState { tileMissing,
+		 tileOld,
+		 tileVeryOld,
+		 tileCurrent
+	       };
 
 #endif
