@@ -18,8 +18,10 @@
 #ifndef PARAMETERIZE_HPP
 #define PARAMETERIZE_HPP
 
-typedef void (*parameterize_function_ptr)(mapnik::Map &m, char * parameter);
+#include <mapnik/map.hpp>
 
-parameterize_function_ptr init_parameterization_function(char * function_name);
+typedef void (*parameterize_function_ptr)(mapnik::Map &m, char *parameter);
+
+parameterize_function_ptr init_parameterization_function(const char *function_name);
 
 #endif
