@@ -15,73 +15,56 @@
  * along with this program; If not, see http://www.gnu.org/licenses/.
  */
 
-#include <mapnik/version.hpp>
-#include <mapnik/map.hpp>
-#include <mapnik/layer.hpp>
-#include <mapnik/datasource.hpp>
-#include <mapnik/feature_type_style.hpp>
-#include <mapnik/datasource_cache.hpp>
-#include <mapnik/agg_renderer.hpp>
-#include <mapnik/load_map.hpp>
-#include <mapnik/image_util.hpp>
-
-#include <exception>
-#include <iostream>
-#include <fstream>
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <sys/time.h>
 #include <dirent.h>
-#include <unistd.h>
-#include <pthread.h>
-#include <string>
-#include <stdlib.h>
-
-#include "gen_tile.h"
-#include "render_config.h"
-#include "daemon.h"
-#include "store.h"
-#include "metatile.h"
-#include "protocol.h"
-#include "request_queue.h"
-#include "cache_expire.h"
-#include "parameterize_style.hpp"
-#include "g_logger.h"
-
-#ifdef HTCP_EXPIRE_CACHE
-#include <sys/socket.h>
-#include <netdb.h>
-#endif
-
-#if MAPNIK_VERSION >= 300000
-#define image_data_32 image_rgba8
-#define image_32 image_rgba8
+#include <exception>
+#include <glib.h>
+#include <map>
+#include <mapnik/agg_renderer.hpp>
+#include <mapnik/coord.hpp>
+#include <mapnik/datasource.hpp>
+#include <mapnik/datasource_cache.hpp>
+#include <mapnik/font_engine_freetype.hpp>
 #include <mapnik/image.hpp>
+#include <mapnik/image_util.hpp>
+#include <mapnik/image_view.hpp>
 #include <mapnik/image_view_any.hpp>
-#else
-#include <mapnik/graphics.hpp>
-#if MAPNIK_VERSION < 200000
-#include <mapnik/envelope.hpp>
-#define image_32 Image32
-#define image_data_32 ImageData32
-#define box2d Envelope
-#define zoom_to_box zoomToBox
-#else
-#if MAPNIK_VERSION >= 400000
+#include <mapnik/layer.hpp>
+#include <mapnik/load_map.hpp>
+#include <mapnik/map.hpp>
+#include <mapnik/params.hpp>
+#include <mapnik/pixel_types.hpp>
+#include <mapnik/version.hpp>
+#include <pthread.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <string>
+#include <sys/time.h>
+#include <unistd.h>
+#include <utility>
+
+#if MAPNIK_MAJOR_VERSION >= 4
 #include <mapnik/geometry/box2d.hpp>
 #else
 #include <mapnik/box2d.hpp>
 #endif
-#endif
-#endif
 
+#include "cache_expire.h"
+#include "g_logger.h"
+#include "gen_tile.h"
+#include "metatile.h"
+#include "parameterize_style.hpp"
+#include "protocol.h"
+#include "render_config.h"
+#include "renderd.h"
+#include "request_queue.h"
+#include "store.h"
 
-using namespace mapnik;
 #ifndef DEG_TO_RAD
-#define DEG_TO_RAD (M_PI/180)
+#define DEG_TO_RAD (M_PI / 180)
 #endif
 #ifndef RAD_TO_DEG
-#define RAD_TO_DEG (180/M_PI)
+#define RAD_TO_DEG (180 / M_PI)
 #endif
 
 #ifdef METATILE
@@ -90,59 +73,61 @@ using namespace mapnik;
 #define RENDER_SIZE (512)
 #endif
 
-extern int foreground;
+using namespace mapnik;
+
+#define image_data_32 image_rgba8
+#define image_32 image_rgba8
 
 struct projectionconfig {
 	double bound_x0;
 	double bound_y0;
 	double bound_x1;
 	double bound_y1;
-	int    aspect_x;
-	int    aspect_y;
+	int aspect_x;
+	int aspect_y;
 };
 
 struct xmlmapconfig {
-	char xmlname[XMLCONFIG_MAX];
-	char xmlfile[PATH_MAX];
-	char output_format[XMLCONFIG_MAX];
-	struct storage_backend * store;
 	Map map;
-	struct projectionconfig * prj;
-	char xmluri[PATH_MAX];
-	char host[PATH_MAX];
-	char htcphost[PATH_MAX];
-	int htcpsock;
-	int tilesize;
+	const char *host;
+	const char *htcphost;
+	const char *output_format;
+	const char *xmlfile;
+	const char *xmlname;
+	const char *xmluri;
 	double scale;
-	int minzoom;
+	int htcpsock;
 	int maxzoom;
+	int minzoom;
 	int ok;
+	int tilesize;
 	parameterize_function_ptr parameterize_function;
-	xmlmapconfig() :
-		map(256, 256) {}
+	struct projectionconfig *prj;
+	struct storage_backend *store;
+
+	xmlmapconfig() : map(256, 256) {}
 };
 
-
-struct projectionconfig * get_projection(const char * srs)
+struct projectionconfig *get_projection(const char *srs)
 {
-	struct projectionconfig * prj;
+	struct projectionconfig *prj;
 
 	if (strstr(srs, "+proj=merc +a=6378137 +b=6378137") != NULL) {
 		g_logger(G_LOG_LEVEL_DEBUG, "Using web mercator projection settings");
 		prj = (struct projectionconfig *)malloc(sizeof(struct projectionconfig));
 		prj->bound_x0 = -20037508.3428;
-		prj->bound_x1 =  20037508.3428;
+		prj->bound_x1 = 20037508.3428;
 		prj->bound_y0 = -20037508.3428;
-		prj->bound_y1 =  20037508.3428;
+		prj->bound_y1 = 20037508.3428;
 		prj->aspect_x = 1;
 		prj->aspect_y = 1;
 	} else if (strcmp(srs, "+proj=eqc +lat_ts=0 +lat_0=0 +lon_0=0 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs") == 0) {
 		g_logger(G_LOG_LEVEL_DEBUG, "Using plate carree projection settings");
 		prj = (struct projectionconfig *)malloc(sizeof(struct projectionconfig));
 		prj->bound_x0 = -20037508.3428;
-		prj->bound_x1 =  20037508.3428;
+		prj->bound_x1 = 20037508.3428;
 		prj->bound_y0 = -10018754.1714;
-		prj->bound_y1 =  10018754.1714;
+		prj->bound_y1 = 10018754.1714;
 		prj->aspect_x = 2;
 		prj->aspect_y = 1;
 	} else if (strcmp(srs, "+proj=tmerc +lat_0=49 +lon_0=-2 +k=0.9996012717 +x_0=400000 +y_0=-100000 +ellps=airy +datum=OSGB36 +units=m +no_defs") == 0) {
@@ -158,9 +143,9 @@ struct projectionconfig * get_projection(const char * srs)
 		g_logger(G_LOG_LEVEL_WARNING, "Unknown projection string, using web mercator as never the less. %s", srs);
 		prj = (struct projectionconfig *)malloc(sizeof(struct projectionconfig));
 		prj->bound_x0 = -20037508.3428;
-		prj->bound_x1 =  20037508.3428;
+		prj->bound_x1 = 20037508.3428;
 		prj->bound_y0 = -20037508.3428;
-		prj->bound_y1 =  20037508.3428;
+		prj->bound_y1 = 20037508.3428;
 		prj->aspect_x = 1;
 		prj->aspect_y = 1;
 	}
@@ -171,45 +156,28 @@ struct projectionconfig * get_projection(const char * srs)
 static void load_fonts(const char *font_dir, int recurse)
 {
 	DIR *fonts = opendir(font_dir);
-	struct dirent *entry;
-	char path[PATH_MAX]; // FIXME: Eats lots of stack space when recursive
 
 	if (!fonts) {
 		g_logger(G_LOG_LEVEL_CRITICAL, "Unable to open font directory: %s", font_dir);
 		return;
 	}
 
-	while ((entry = readdir(fonts))) {
-		struct stat b;
-		char *p;
-
-		if (!strcmp(entry->d_name, ".") || !strcmp(entry->d_name, "..")) {
-			continue;
-		}
-
-		snprintf(path, sizeof(path), "%s/%s", font_dir, entry->d_name);
-
-		if (stat(path, &b)) {
-			continue;
-		}
-
-		if (S_ISDIR(b.st_mode)) {
-			if (recurse) {
-				load_fonts(path, recurse);
-			}
-
-			continue;
-		}
-
-		p = strrchr(path, '.');
-
-		if (p && (!strcmp(p, ".ttf") || !strcmp(p, ".otf") || !strcmp(p, ".ttc"))) {
-			g_logger(G_LOG_LEVEL_DEBUG, "Loading font: %s", path);
-			freetype_engine::register_font(path);
-		}
-	}
-
 	closedir(fonts);
+
+	g_logger(G_LOG_LEVEL_INFO, "Loading fonts%sfrom directory '%s'", recurse ? " recursively " : " ", font_dir);
+
+	freetype_engine::register_fonts(font_dir, recurse);
+
+	freetype_engine::font_file_mapping_type face_name_mappings = freetype_engine::get_mapping();
+
+	freetype_engine::font_file_mapping_type::iterator face_name_mappings_iterator = face_name_mappings.begin();
+
+	while (face_name_mappings_iterator != face_name_mappings.end()) {
+		const char *face_name = face_name_mappings_iterator->first.c_str();
+		const char *font_file_name = face_name_mappings_iterator->second.second.c_str();
+		g_logger(G_LOG_LEVEL_DEBUG, "Loaded font face '%s' from file '%s'", face_name, font_file_name);
+		face_name_mappings_iterator++;
+	}
 }
 
 /**
@@ -219,33 +187,24 @@ static void load_fonts(const char *font_dir, int recurse)
 static void parameterize_map_max_connections(Map &m, int num_threads)
 {
 	unsigned int i;
-	char * tmp = (char *)malloc(20);
+	char *tmp = (char *)malloc(20);
 
 	for (i = 0; i < m.layer_count(); i++) {
-#if MAPNIK_VERSION >= 300000
-		layer& l = m.get_layer(i);
-#else
-		layer& l = m.getLayer(i);
-#endif
+		layer &l = m.get_layer(i);
 		parameters params = l.datasource()->params();
 
 		if (params.find("max_size") == params.end()) {
-			sprintf(tmp, "%i", num_threads + 2);
+			snprintf(tmp, 20, "%i", num_threads + 2);
 			params["max_size"] = std::string(tmp);
 		}
 
-#if MAPNIK_VERSION >= 200200
 		l.set_datasource(datasource_cache::instance().create(params));
-#else
-		l.set_datasource(datasource_cache::instance()->create(params));
-#endif
 	}
 
 	free(tmp);
 }
 
-
-static int check_xyz(int x, int y, int z, struct xmlmapconfig * map)
+static int check_xyz(int x, int y, int z, struct xmlmapconfig *map)
 {
 	int oob, limit;
 
@@ -266,7 +225,7 @@ static int check_xyz(int x, int y, int z, struct xmlmapconfig * map)
 }
 
 #ifdef METATILE
-mapnik::box2d<double> tile2prjbounds(struct projectionconfig * prj, int x, int y, int z)
+mapnik::box2d<double> tile2prjbounds(struct projectionconfig *prj, int x, int y, int z)
 {
 
 	int render_size_tx = MIN(METATILE, prj->aspect_x * (1 << z));
@@ -280,10 +239,10 @@ mapnik::box2d<double> tile2prjbounds(struct projectionconfig * prj, int x, int y
 	g_logger(G_LOG_LEVEL_DEBUG, "Rendering projected coordinates %i %i %i -> %f|%f %f|%f to a %i x %i tile", z, x, y, p0x, p0y, p1x, p1y, render_size_tx, render_size_ty);
 
 	mapnik::box2d<double> bbox(p0x, p0y, p1x, p1y);
-	return  bbox;
+	return bbox;
 }
 
-static enum protoCmd render(struct xmlmapconfig * map, int x, int y, int z, char *options, metaTile &tiles)
+static enum protoCmd render(struct xmlmapconfig *map, int x, int y, int z, char *options, metaTile &tiles)
 {
 	unsigned int render_size_tx = MIN(METATILE, map->prj->aspect_x * (1 << z));
 	unsigned int render_size_ty = MIN(METATILE, map->prj->aspect_y * (1 << z));
@@ -295,7 +254,7 @@ static enum protoCmd render(struct xmlmapconfig * map, int x, int y, int z, char
 		map->map.set_buffer_size((map->tilesize >> 1) * map->scale);
 	}
 
-	//m.zoom(size+1);
+	// m.zoom(size+1);
 
 	mapnik::image_32 buf(render_size_tx * map->tilesize, render_size_ty * map->tilesize);
 
@@ -313,7 +272,7 @@ static enum protoCmd render(struct xmlmapconfig * map, int x, int y, int z, char
 			mapnik::agg_renderer<mapnik::image_32> ren(map->map, buf, map->scale);
 			ren.apply();
 		}
-	} catch (std::exception const& ex) {
+	} catch (std::exception const &ex) {
 		g_logger(G_LOG_LEVEL_ERROR, "failed to render TILE %s %d %d-%d %d-%d", map->xmlname, z, x, x + render_size_tx - 1, y, y + render_size_ty - 1);
 		g_logger(G_LOG_LEVEL_ERROR, "  reason: %s", ex.what());
 		return cmdNotDone;
@@ -324,20 +283,16 @@ static enum protoCmd render(struct xmlmapconfig * map, int x, int y, int z, char
 
 	for (yy = 0; yy < render_size_ty; yy++) {
 		for (xx = 0; xx < render_size_tx; xx++) {
-#if MAPNIK_VERSION >= 300000
 			mapnik::image_view<mapnik::image<mapnik::rgba8_t>> vw1(xx * map->tilesize, yy * map->tilesize, map->tilesize, map->tilesize, buf);
 			struct mapnik::image_view_any vw(vw1);
-#else
-			mapnik::image_view<mapnik::image_data_32> vw(xx * map->tilesize, yy * map->tilesize, map->tilesize, map->tilesize, buf.data());
-#endif
 			tiles.set(xx, yy, save_to_string(vw, map->output_format));
 		}
 	}
 
 	return cmdDone; // OK
 }
-#else //METATILE
-static enum protoCmd render(Map &m, const char *tile_dir, char *xmlname, projection &prj, int x, int y, int z, char* outputFormat)
+#else  // METATILE
+static enum protoCmd render(Map &m, const char *tile_dir, char *xmlname, projection &prj, int x, int y, int z, char *outputFormat)
 {
 	char filename[PATH_MAX];
 	char tmp[PATH_MAX];
@@ -380,43 +335,38 @@ static enum protoCmd render(Map &m, const char *tile_dir, char *xmlname, project
 
 	return cmdDone; // OK
 }
-#endif //METATILE
+#endif // METATILE
 
-
-void render_init(const char *plugins_dir, const char* font_dir, int font_dir_recurse)
+void render_init(const char *plugins_dir, const char *font_dir, int font_dir_recurse)
 {
-	g_logger(G_LOG_LEVEL_INFO, "Renderd is using mapnik version %i.%i.%i", ((MAPNIK_VERSION) / 100000), (((MAPNIK_VERSION) / 100) % 1000), ((MAPNIK_VERSION) % 100));
-
-#if MAPNIK_VERSION >= 200200
+	g_logger(G_LOG_LEVEL_INFO, "Renderd is using mapnik version %i.%i.%i", MAPNIK_MAJOR_VERSION, MAPNIK_MINOR_VERSION, MAPNIK_PATCH_VERSION);
 	mapnik::datasource_cache::instance().register_datasources(plugins_dir);
-#else
-	mapnik::datasource_cache::instance()->register_datasources(plugins_dir);
-#endif
 	load_fonts(font_dir, font_dir_recurse);
 }
 
-void *render_thread(void * arg)
+void *render_thread(void *arg)
 {
-	xmlconfigitem * parentxmlconfig = (xmlconfigitem *)arg;
+	xmlconfigitem *parentxmlconfig = (xmlconfigitem *)arg;
 	xmlmapconfig maps[XMLCONFIGS_MAX];
 	int i, iMaxConfigs;
 	int render_time;
 
+	g_logger(G_LOG_LEVEL_DEBUG, "Starting rendering thread: %lu", (unsigned long)pthread_self());
+
 	for (iMaxConfigs = 0; iMaxConfigs < XMLCONFIGS_MAX; ++iMaxConfigs) {
-		if (parentxmlconfig[iMaxConfigs].xmlname[0] == 0 || parentxmlconfig[iMaxConfigs].xmlfile[0] == 0) {
+		if (parentxmlconfig[iMaxConfigs].xmlname == NULL || parentxmlconfig[iMaxConfigs].xmlfile == NULL) {
 			break;
 		}
 
-		strcpy(maps[iMaxConfigs].xmlname, parentxmlconfig[iMaxConfigs].xmlname);
-		strcpy(maps[iMaxConfigs].xmlfile, parentxmlconfig[iMaxConfigs].xmlfile);
-		strcpy(maps[iMaxConfigs].output_format, parentxmlconfig[iMaxConfigs].output_format);
-		maps[iMaxConfigs].store = init_storage_backend(parentxmlconfig[iMaxConfigs].tile_dir);
-		maps[iMaxConfigs].tilesize  = parentxmlconfig[iMaxConfigs].tile_px_size;
-		maps[iMaxConfigs].scale  = parentxmlconfig[iMaxConfigs].scale_factor;
-		maps[iMaxConfigs].minzoom = parentxmlconfig[iMaxConfigs].min_zoom;
 		maps[iMaxConfigs].maxzoom = parentxmlconfig[iMaxConfigs].max_zoom;
+		maps[iMaxConfigs].minzoom = parentxmlconfig[iMaxConfigs].min_zoom;
+		maps[iMaxConfigs].output_format = strndup(parentxmlconfig[iMaxConfigs].output_format, PATH_MAX);
 		maps[iMaxConfigs].parameterize_function = init_parameterization_function(parentxmlconfig[iMaxConfigs].parameterization);
-
+		maps[iMaxConfigs].scale = parentxmlconfig[iMaxConfigs].scale_factor;
+		maps[iMaxConfigs].store = init_storage_backend(parentxmlconfig[iMaxConfigs].tile_dir);
+		maps[iMaxConfigs].tilesize = parentxmlconfig[iMaxConfigs].tile_px_size;
+		maps[iMaxConfigs].xmlfile = strndup(parentxmlconfig[iMaxConfigs].xmlfile, PATH_MAX);
+		maps[iMaxConfigs].xmlname = strndup(parentxmlconfig[iMaxConfigs].xmlname, PATH_MAX);
 
 		if (maps[iMaxConfigs].store) {
 			maps[iMaxConfigs].ok = 1;
@@ -434,12 +384,12 @@ void *render_thread(void * arg)
 				 * up the mapnik datasources to support larger postgres connection pools
 				 */
 				if (parentxmlconfig[iMaxConfigs].num_threads > 10) {
-					g_logger(G_LOG_LEVEL_INFO, "Updating max_connection parameter for mapnik layers to reflect thread count");
+					g_logger(G_LOG_LEVEL_DEBUG, "Updating max_connection parameter for mapnik layers to reflect thread count");
 					parameterize_map_max_connections(maps[iMaxConfigs].map, parentxmlconfig[iMaxConfigs].num_threads);
 				}
 
 				maps[iMaxConfigs].prj = get_projection(maps[iMaxConfigs].map.srs().c_str());
-			} catch (std::exception const& ex) {
+			} catch (std::exception const &ex) {
 				g_logger(G_LOG_LEVEL_ERROR, "An error occurred while loading the map layer '%s': %s", maps[iMaxConfigs].xmlname, ex.what());
 				maps[iMaxConfigs].ok = 0;
 			} catch (...) {
@@ -448,16 +398,15 @@ void *render_thread(void * arg)
 			}
 
 #ifdef HTCP_EXPIRE_CACHE
-			strcpy(maps[iMaxConfigs].xmluri, parentxmlconfig[iMaxConfigs].xmluri);
-			strcpy(maps[iMaxConfigs].host, parentxmlconfig[iMaxConfigs].host);
-			strcpy(maps[iMaxConfigs].htcphost, parentxmlconfig[iMaxConfigs].htcpip);
+			maps[iMaxConfigs].host = strndup(parentxmlconfig[iMaxConfigs].host, PATH_MAX);
+			maps[iMaxConfigs].htcphost = strndup(parentxmlconfig[iMaxConfigs].htcpip, PATH_MAX);
+			maps[iMaxConfigs].xmluri = strndup(parentxmlconfig[iMaxConfigs].xmluri, PATH_MAX);
 
 			if (strlen(maps[iMaxConfigs].htcphost) > 0) {
-				maps[iMaxConfigs].htcpsock = init_cache_expire(
-								     maps[iMaxConfigs].htcphost);
+				maps[iMaxConfigs].htcpsock = init_cache_expire(maps[iMaxConfigs].htcphost);
 
 				if (maps[iMaxConfigs].htcpsock > 0) {
-					g_logger(G_LOG_LEVEL_INFO, "Successfully opened socket for HTCP cache expiry");
+					g_logger(G_LOG_LEVEL_DEBUG, "Successfully opened socket for HTCP cache expiry");
 				} else {
 					g_logger(G_LOG_LEVEL_ERROR, "Failed to open socket for HTCP cache expiry");
 				}
@@ -465,7 +414,7 @@ void *render_thread(void * arg)
 				maps[iMaxConfigs].htcpsock = -1;
 			}
 
-#endif
+#endif // HTCP_EXPIRE_CACHE
 		} else {
 			maps[iMaxConfigs].ok = 0;
 		}
@@ -518,9 +467,9 @@ void *render_thread(void * arg)
 									tiles.save(maps[i].store);
 #ifdef HTCP_EXPIRE_CACHE
 									tiles.expire_tiles(maps[i].htcpsock, maps[i].host, maps[i].xmluri);
-#endif
+#endif // HTCP_EXPIRE_CACHE
 
-								} catch (std::exception const& ex) {
+								} catch (std::exception const &ex) {
 									g_logger(G_LOG_LEVEL_ERROR, "Received exception when writing metatile to disk: %s", ex.what());
 									ret = cmdNotDone;
 								} catch (...) {
@@ -531,12 +480,12 @@ void *render_thread(void * arg)
 								}
 							}
 
-#else //METATILE
+#else // METATILE
 			ret = render(maps[i].map, maps[i].tile_dir, req->xmlname, maps[i].prj, req->x, req->y, req->z, maps[i].output_format);
 #ifdef HTCP_EXPIRE_CACHE
 			cache_expire(maps[i].htcpsock, maps[i].host, maps[i].xmluri, req->x, req->y, req->z);
-#endif
-#endif //METATILE
+#endif // HTCP_EXPIRE_CACHE
+#endif // METATILE
 						} else {
 							g_logger(G_LOG_LEVEL_WARNING, "Received request for map layer %s is outside of acceptable bounds z(%i), x(%i), y(%i)",
 								 req->xmlname, req->z, req->x, req->y);
@@ -550,7 +499,7 @@ void *render_thread(void * arg)
 					send_response(item, ret, render_time);
 
 					if ((ret != cmdDone) && (ret != cmdIgnore)) {
-						sleep(10);        //Something went wrong with rendering, delay next processing to allow temporary issues to fix them selves
+						sleep(10); // Something went wrong with rendering, delay next processing to allow temporary issues to fix them selves
 					}
 
 					break;
