@@ -15,19 +15,18 @@
  * along with this program; If not, see http://www.gnu.org/licenses/.
  */
 
-#include "config.h"
-
-#include <string.h>
+#include <glib.h>
 #include <limits.h>
 #include <stdlib.h>
+#include <string.h>
+#include <string>
+#include <sys/types.h>
 
-#include "render_config.h"
-#include "metatile.h"
-#include "store.h"
 #include "cache_expire.h"
-#include "request_queue.h"
 #include "g_logger.h"
-
+#include "metatile.h"
+#include "render_config.h"
+#include "store.h"
 
 metaTile::metaTile(const std::string &xmlconfig, const std::string &options, int x, int y, int z):
 	x_(x), y_(y), z_(z), xmlconfig_(xmlconfig), options_(options)
@@ -122,7 +121,7 @@ void metaTile::save(struct storage_backend * store)
 }
 
 
-void metaTile::expire_tiles(int sock, char * host, char * uri)
+void metaTile::expire_tiles(int sock, const char *host, const char *uri)
 {
 	if (sock < 0) {
 		return;
